@@ -13,14 +13,20 @@ import os
 import sys
 import stat
 import json
-import string
 import shutil
+import filecmp
+import logging
 import getpass
 import datetime
+import traceback
 import subprocess
 from tempfile import mkstemp
 from shutil import move
 
+from tpDcc.libs.python import python
+
+logger = logging.getLogger('tpDcc-libs-python')
+
 
 class FileManager(object):
     """
@@ -60,7 +66,7 @@ class FileManager(object):
         Opens managed file to write data into (removing any previous data)
         """
 
-        self.check_file(warning_text='File {} is invalid!'.format(self.file_path))
+        # self.check_file(warning_text='File {} is invalid!'.format(self.file_path))
         self.open_file = open(self.file_path, 'w')
 
     def append_file(self):
@@ -213,9 +219,12 @@ class FileWriter(FileManager, object):
 
         self.write_file()
         try:
-            json.dump(dict_data, self.open_file, indent=4, sort_keys=True)
-        except Exception:
-            pass
+            if python.is_python2():
+                json.dump(dict_data, self.open_file, indent=4, sort_keys=False)
+            else:
+                json.dump(dict(dict_data), self.open_file, indent=4, sort_keys=False)
+        except Exception as exc:
+            logger.exception('Impossible to save JSON file: "{}"'.format(exc))
         self.close_file()
 
     def write(self, lines, last_line_empty=True):
@@ -409,8 +418,7 @@ class FileVersion(object):
 
         if not comment:
             comment = '-'
-        comment.replace('\n', '   ')
-        comment.replace('\r', '   ')
+        comment = comment.replace('\n', '   ').replace('\r', '   ')
 
         self._create_version_folder()
         self._create_comment_file()
@@ -603,8 +611,6 @@ def open_browser(file_path):
     :return:
     """
 
-    # TODO: Only work on Windows, make it cross platform
-
     from tpDcc.libs.python import osplatform, path
 
     if not path.is_file(file_path) and not path.is_dir(file_path):
@@ -614,13 +620,16 @@ def open_browser(file_path):
         os.startfile(file_path)
     elif osplatform.is_linux():
         try:
-            opener = 'open' if sys.platform == 'darwin' else 'xdg-open'
-            subprocess.call([opener, file_path])
-        except Exception as e:
-            os.system('gnome-terminal --working-directory={}'.format(file_path))
+            os.system('gio open {}'.format(file_path))
+        except Exception:
+            try:
+                opener = 'open' if sys.platform == 'darwin' else 'xdg-open'
+                subprocess.call([opener, file_path])
+            except Exception as e:
+                os.system('gnome-terminal --working-directory={}'.format(file_path))
 
 
-def create_file(filename, directory, make_unique=False):
+def create_file(filename, directory=None, make_unique=False):
     """
     Creates a file
     :param filename: str, name of the new file
@@ -629,7 +638,11 @@ def create_file(filename, directory, make_unique=False):
     :return: variant, str || bool, filename with path or False if create file failed
     """
 
-    from tpDcc.libs.python import name, path
+    from tpDcc.libs.python import name, path, osplatform
+
+    if directory is None:
+        directory = path.get_dirname(filename)
+        filename = path.get_basename(filename)
 
     filename = name.clean_file_string(filename)
     full_path = path.join_path(directory, filename)
@@ -637,12 +650,17 @@ def create_file(filename, directory, make_unique=False):
     if make_unique:
         full_path = path.unique_path_name(full_path)
 
+    open_file = None
     try:
         open_file = open(full_path, 'a')
         open_file.close()
     except Exception:
+        if open_file:
+            open_file.close()
         return False
 
+    osplatform.get_permission(full_path)
+
     return full_path
 
 
@@ -654,7 +672,16 @@ def copy_file(file_path, file_path_destination):
     :return: str, the new copied path
     """
 
-    shutil.copy2(file_path, file_path_destination)
+    from tpDcc.libs.python import path, osplatform
+
+    osplatform.get_permission(file_path)
+
+    if path.is_file(file_path):
+        if path.is_dir(file_path_destination):
+            file_name = path.get_basename(file_path)
+            file_path_destination = path.join_path(file_path_destination, file_name)
+        shutil.copy2(file_path, file_path_destination)
+
     return file_path_destination
 
 
@@ -675,22 +702,34 @@ def move_file(path1, path2):
     return True
 
 
-def delete_file(name, directory):
+def delete_file(name, directory=None, show_warning=True):
     """
     Delete the file by name in the directory
     :param name: str, name of the file to delete
     :param directory: str, the directory where the file is stored
+    :param show_warning: bool
     :return: str, file path that was deleted
     """
 
-    from tpDcc.libs.python import path
+    from tpDcc.libs.python import path, osplatform
 
-    full_path = path.join_path(directory, name)
+    if not directory:
+        full_path = name
+    else:
+        full_path = path.join_path(directory, name)
     if not path.is_file(full_path):
+        if show_warning:
+            print('File "{}" was not deleted.'.format(full_path))
         return full_path
 
-    os.chmod(full_path, 0o777)
-    os.remove(full_path)
+    try:
+        osplatform.get_permission(full_path)
+    except Exception:
+        pass
+    try:
+        os.remove(full_path)
+    except Exception:
+        pass
 
     return full_path
 
@@ -728,12 +767,11 @@ def write_to_file(file_name, text_to_write):
     """
 
     if os.path.exists(file_name):
-        readOnlyOrWriteable = os.stat(file_name)[0]
-        if readOnlyOrWriteable != stat.S_IWRITE:
+        read_only_or_writeable = os.stat(file_name)[0]
+        if read_only_or_writeable != stat.S_IWRITE:
             os.chmod(file_name, stat.S_IWRITE)
-    file = open(file_name, 'w')
-    file.write(text_to_write)
-    file.close()
+    with open(file_name, 'w') as file:
+        file.write(text_to_write)
 
 
 def append_to_file(file_name, text_to_add):
@@ -745,9 +783,8 @@ def append_to_file(file_name, text_to_add):
         readOnlyOrWriteable = os.stat(file_name)[0]
         if readOnlyOrWriteable != stat.S_IWRITE:
             os.chmod(file_name, stat.S_IWRITE)
-    file = open(file_name, 'a')
-    file.write(text_to_add)
-    file.close()
+    with open(file_name, 'a') as file:
+        file.write(text_to_add)
 
 
 def replace(file_path, pattern, subst):
@@ -786,7 +823,7 @@ def remove_extension(file_path):
     split_path = file_path.split('.')
     new_name = file_path
     if len(split_path) > 1:
-        new_name = string.join(split_path[:-1], '.')
+        new_name = '.'.join(split_path[:-1])
 
     return new_name
 
@@ -821,6 +858,19 @@ def is_file_in_dir(filename, directory):
     return os.path.isfile(file_path)
 
 
+def is_file_empty(file_path):
+    """
+    Returns whether or not given file is empty
+    :param file_path: str
+    :return: bool, True if file is empty; False otherwise.
+    """
+
+    if not os.path.isfile(file_path):
+        return True
+
+    return os.stat(file_path).st_size == 0
+
+
 def is_same_date(file1, file2):
     """
     Returns True if the given files have the same date
@@ -829,12 +879,21 @@ def is_same_date(file1, file2):
     :return: bool
     """
 
+    if not file1 and file2 or file1 and not file2:
+        return False
+    if not file1 and not file2:
+        return True
+
     date1 = os.path.getmtime(file1)
     date2 = os.path.getmtime(file2)
-    value = date1 - date2
-    if abs(value) < 0.01:
+    if not date1 and not date2:
         return True
 
+    if date1 and date2:
+        value = date1 - date2
+        if abs(value) < 0.01:
+            return True
+
     return False
 
 
@@ -845,14 +904,11 @@ def get_file_text(file_path):
     :return: str
     """
 
-    open_file = open(file_path, 'r')
-    lines = ''
-
     try:
-        lines = open_file.read()
-        open_file.close()
+        with open(file_path, 'r') as open_file:
+            lines = open_file.read()
     except Exception:
-        open_file.close()
+        return list()
 
     return lines
 
@@ -890,11 +946,12 @@ def get_size(file_path, round_value=2):
     return size
 
 
-def get_last_modified_date(file_path):
+def get_last_modified_date(file_path, reverse_date=False):
     """
     Returns the last date given file was modified
     :param file_path: str
-    :return: str, formated date and time
+    :param reverse_date: bool
+    :return: str, formatted date and time
     """
 
     mtime = os.path.getatime(file_path)
@@ -915,7 +972,10 @@ def get_last_modified_date(file_path):
     if len(second) == 1:
         second = second + '0'
 
-    return '{0}-{1}-{2}  {3}:{4}:{5}'.format(year, month, day, hour, minute, second)
+    if reverse_date:
+        return '{0}-{1}-{2}  {3}:{4}:{5}'.format(year, month, day, hour, minute, second)
+    else:
+        return '{0}-{1}-{2}  {3}:{4}:{5}'.format(day, month, year, hour, minute, second)
 
 
 def get_file_date(file_path):
@@ -952,8 +1012,11 @@ def get_file_lines(file_path):
     :return: str
     """
 
-    reader = FileReader(file_path=file_path)
-    return reader.read()
+    text = get_file_text(file_path)
+    if not text:
+        return list()
+
+    return get_text_lines(text)
 
 
 def get_text_lines(text):
@@ -969,6 +1032,20 @@ def get_text_lines(text):
     return lines
 
 
+def write_replace(file_path, data_to_write):
+    """
+    Writes given data into given file path (replacing already existing content)
+    :param file_path: str
+    :param data_to_write:
+    """
+
+    with open(file_path, 'w') as file_handle:
+        try:
+            file_handle.write(data_to_write)
+        except Exception:
+            logger.warning('Could not write: {} | {}'.format(data_to_write, traceback.format_exc()))
+
+
 def write_lines(file_path, lines, append=False):
     """
     Writes a list of text lines to a file. Every entry in the list is a new line
@@ -977,10 +1054,21 @@ def write_lines(file_path, lines, append=False):
     :param append: bool, Whether to append the text or replace it
     """
 
-    write_file = FileWriter(file_path=file_path)
-    write_file.set_append(append)
-    write_file.write(lines)
-    write_file.close_file()
+    from tpDcc.libs.python import python, osplatform
+
+    permission = osplatform.get_permission(file_path)
+    if not permission:
+        return
+
+    write_string = 'a' if append else 'w'
+
+    lines = python.force_list(lines)
+    text = '\n'.join(map(str, lines))
+    if append:
+        text = '\n' + text
+
+    with open(file_path, write_string) as open_file:
+        open_file.write(text)
 
 
 def is_same_text_content(file1, file2):
@@ -991,18 +1079,14 @@ def is_same_text_content(file1, file2):
     :return: bool
     """
 
-    text1 = get_file_text(file_path=file1)
-    text2 = get_file_text(file_path=file2)
-    if text1 == text2:
-        return True
-
-    return False
+    return filecmp.cmp(file1, file2)
 
 
-def get_files(root_directory):
+def get_files(root_directory, filter_text=''):
     """
-    Returns files found in the given directorys
+    Returns files found in the given directory
     :param root_directory: str
+    :param filter_text: str
     :return: list(str)
     """
 
@@ -1010,6 +1094,8 @@ def get_files(root_directory):
 
     files = os.listdir(root_directory)
     for filename in files:
+        if filter_text and filename.find(filter_text) == -1:
+            continue
         file_path = os.path.join(root_directory, filename)
         if os.path.isfile(file_path):
             found.append(filename)
@@ -1029,3 +1115,77 @@ def file_has_info(file_path):
         return False
 
     return True
+
+
+def get_lock_name(file_path):
+    """
+    Returns lock file of the given file
+    :param file_path: str
+    :return: str
+    """
+
+    return '{}.lock'.format(file_path)
+
+
+def is_locked(file_path):
+    """
+    Returns whether or not given file is locked
+    :param file_path: str
+    :return: bool
+    """
+
+    return os.path.isfile(get_lock_name(file_path))
+
+
+def lock(file_path):
+    """
+    Creates lock file of the given file
+    :param file_path: str
+    :return: str
+    """
+
+    lock_file = get_lock_name(file_path)
+    create_file(lock_file)
+
+    return lock_file
+
+
+def remove_lock(file_path):
+    """
+    Removes lock file of the given file
+    :param file_path: str
+    :return: bool
+    """
+
+    lock_file = get_lock_name(file_path)
+    if not os.path.isfile(lock_file):
+        return False
+
+    return delete_file(lock_file)
+
+
+def get_latest_file(file_paths, only_return_one_match=True):
+    """
+    Returns the latest created file from a list of file paths
+    :param file_paths: list(str)
+    :param only_return_one_match: bool
+    :return: list(str) or str
+    """
+    last_time = 0
+    times = dict()
+
+    for file_path in file_paths:
+        mtime = os.stat(file_path).st_mtime
+        if mtime not in times:
+            times[mtime] = list()
+        times[mtime].append(file_path)
+        if mtime > last_time:
+            last_time = mtime
+
+    if not times:
+        return
+
+    if only_return_one_match:
+        return times[mtime][0]
+    else:
+        return times[mtime]
