@@ -17,9 +17,7 @@ import tempfile
 import traceback
 import contextlib
 
-import six
-
-from tpDcc.libs.python import name, folder
+from tpDcc.libs.python import name, folder, osplatform, python, win32
 
 SEPARATOR = '/'
 BAD_SEPARATOR = '\\'
@@ -32,7 +30,7 @@ WEB_PREFIX = 'https://'
 # We use one separator depending if we are working on Windows (nt) or other operative system
 NATIVE_SEPARATOR = (SEPARATOR, BAD_SEPARATOR)[os.name == 'nt']
 
-LOGGER = logging.getLogger()
+logger = logging.getLogger('tpDcc-libs-python')
 
 
 class FindUniquePath(name.FindUniqueString, object):
@@ -85,7 +83,13 @@ def normalize_path(path):
     """
 
     path = path.replace(BAD_SEPARATOR, SEPARATOR).replace(PATH_SEPARATOR, SEPARATOR)
-    path = six.u(str(path))
+
+    if python.is_python2():
+        try:
+            path = unicode(path.replace(r'\\', r'\\\\'), "unicode_escape").encode('utf-8')
+        except TypeError:
+            path = path.replace(r'\\', r'\\\\').encode('utf-8')
+
     return path.rstrip('/')
 
 
@@ -106,6 +110,9 @@ def clean_path(path):
     :return: str, clean path
     """
 
+    if not path:
+        return
+
     # We convert '~' Unix character to user's home directory
     path = os.path.expanduser(str(path))
 
@@ -147,18 +154,28 @@ def join_path(dir1, dir2):
     :return: str, combined directory path
     """
 
-    if not dir1 or not dir2:
+    if dir1 is None or dir2 is None:
         return None
 
     dir1 = clean_path(dir1)
     dir2 = clean_path(dir2)
 
-    dir_path = '{0}/{1}'.format(dir1, dir2)
+    dir_path = '{}/{}'.format(dir1, dir2) if dir2 else dir1
     dir_path = clean_path(dir_path)
 
     return dir_path
 
 
+def set_windows_slashes(directory):
+    """
+    Set all the slashes in a name so they use Windows slashes (\)
+    :param directory: str
+    :return: str
+    """
+
+    return directory.replace('/', '\\').replace('//', '\\')
+
+
 def split_path(path):
     """
     Split the given path into directory, basename and extension
@@ -264,6 +281,16 @@ def exists(directory):
     :return: bool
     """
 
+    if not directory:
+        return False
+
+    try:
+        stat = os.stat(directory)
+        if stat:
+            return True
+    except Exception:
+        return False
+
     return os.path.exists(directory)
 
 
@@ -452,7 +479,7 @@ def remove_common_path(path1, path2):
         if not skip:
             new_path.append(split_path2[i])
 
-    new_path = string.join(new_path, '/')
+    new_path = '/'.join(new_path)
 
     return new_path
 
@@ -483,6 +510,7 @@ def is_dir(directory, path=None):
     """
     Checks if the given directory is a directory or not
     :param directory: str
+    :param path: str
     :return: bool
     """
 
@@ -536,7 +564,7 @@ def move(path1, path2):
     try:
         shutil.move(path1, path2)
     except Exception:
-        LOGGER.warning('Failed to move {0} to {1}'.format(path1, path2))
+        logger.warning('Failed to move {0} to {1}'.format(path1, path2))
         return False
 
     return True
@@ -567,10 +595,64 @@ def rename(directory, name, make_unique=False):
     try:
         os.chmod(directory, 0o777)
         message = 'rename: {0} >> {1}'.format(directory, rename_path)
-        LOGGER.info(message)
+        logger.info(message)
         os.rename(directory, rename_path)
     except Exception:
-        LOGGER.error('{}'.format(traceback.format_exc()))
+        logger.error('{}'.format(traceback.format_exc()))
         return False
 
     return rename_path
+
+
+def get_user_data_dir(appname=None, appauthor=None, version=None, roaming=False):
+    r"""
+    Based on appdirs user_data_dir function
+
+    Returns the full path to the user-specific data directory
+    :param appname: str or None, name of the application. If None, the system directory is returned
+    :param appauthor: str, name of the author or distributing body for this application. Typically is the owning
+        company name. Only used in Windows.
+    :param version: str, optional version path element to append to the path. You might want to use this if you
+        want multiple versions of you app to be able to run independently. If used, this would typically be
+        "<major>.<minor>". Only is applied if app_name is present.
+    :param roaming: bool, True to use the Windows roaming appdata directory. That means that for users on a a
+        Windows network setup for roaming profiles, this user data will be synced on login. See
+        <http://technet.microsoft.com/en-us/library/cc766489(WS.10).aspx> for a discussion of issues.
+    :return: str
+
+    Typical user data directories are:
+        Mac OS X:               ~/Library/Application Support/<AppName>
+        Unix:                   ~/.local/share/<AppName>    # or in $XDG_DATA_HOME, if defined
+        Win XP (not roaming):   C:\Documents and Settings\<username>\Application Data\<AppAuthor>\<AppName>
+        Win XP (roaming):       C:\Documents and Settings\<user>\Local Settings\Application Data\<AppAuthor>\<AppName>
+        Win 7  (not roaming):   C:\Users\<username>\AppData\Local\<AppAuthor>\<AppName>
+        Win 7  (roaming):       C:\Users\<username>\AppData\Roaming\<AppAuthor>\<AppName>
+
+    For Unix, we follow the XDG spec and support $XDG_DATA_HOME.
+    That means, by default "~/.local/share/<AppName>".
+    """
+
+    system = osplatform.get_sys_platform()
+
+    if system == "win32":
+        if appauthor is None:
+            appauthor = appname
+        const = roaming and "CSIDL_APPDATA" or "CSIDL_LOCAL_APPDATA"
+        path = os.path.normpath(win32.get_win_folder(const))
+        if appname:
+            if appauthor is not False:
+                path = os.path.join(path, appauthor, appname)
+            else:
+                path = os.path.join(path, appname)
+    elif system == 'darwin':
+        path = os.path.expanduser('~/Library/Application Support/')
+        if appname:
+            path = os.path.join(path, appname)
+    else:
+        path = os.getenv('XDG_DATA_HOME', os.path.expanduser("~/.local/share"))
+        if appname:
+            path = os.path.join(path, appname)
+    if appname and version:
+        path = os.path.join(path, version)
+
+    return path
