@@ -11,7 +11,7 @@ import logging
 
 from tpDcc.libs.python import folder, path, fileio, jsonio, sort, name as name_utils
 
-LOGGER = logging.getLogger('tpDcc-libs-python')
+logger = logging.getLogger('tpDcc-libs-python')
 
 
 class SemanticVersion(object):
@@ -258,7 +258,7 @@ class VersionFile(object):
         if file_name:
             self._save(file_name)
         else:
-            LOGGER.warning('Could not save default version!')
+            logger.warning('Could not save default version!')
 
         return file_name
 
@@ -283,7 +283,7 @@ class VersionFile(object):
         version_folder = self._get_version_folder()
         version_files = folder.get_files_and_folders(version_folder)
         if not version_files:
-            LOGGER.warning('Impossible to get versions because no version exist!')
+            logger.warning('Impossible to get versions because no version exist!')
             return None
 
         number_list = list()
@@ -298,7 +298,7 @@ class VersionFile(object):
             pass_files.append(f)
 
         if not pass_files:
-            LOGGER.warning('No valid version files found in folder: {}'.format(version_folder))
+            logger.warning('No valid version files found in folder: {}'.format(version_folder))
             return
 
         quick_sort = sort.QuickNumbersListSort(number_list)
@@ -323,7 +323,7 @@ class VersionFile(object):
         version_folder = self._get_version_folder()
         version_files = folder.get_files_and_folders(version_folder)
         if not version_files:
-            LOGGER.warning('Impossible to get version numbers because no version exist!')
+            logger.warning('Impossible to get version numbers because no version exist!')
             return None
 
         number_list = list()
